package cc.fascinated.bat.features.logging.listeners;

import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.logging.LogFeature;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.Invite;
import net.dv8tion.jda.api.entities.channel.ChannelType;
import net.dv8tion.jda.api.events.guild.invite.GuildInviteCreateEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2(topic = "Logging - Guild Listener")
public class GuildListener implements EventListener {
    private final LogFeature logFeature;

    @Autowired
    public GuildListener(@NonNull ApplicationContext context) {
        this.logFeature = context.getBean(LogFeature.class);
    }

    @Override
    public void onGuildInviteCreate(@NonNull BatGuild guild, @NonNull Invite invite, @NonNull GuildInviteCreateEvent event) {
        log.info("Invite created in guild \"{}\" with code \"{}\"", guild.getGuild().getName(), invite.getCode());

        invite.expand().queue(expandedInvite -> {
            EmbedDescriptionBuilder description = new EmbedDescriptionBuilder("Invite Created");
            description.appendLine("Invite: %s".formatted(expandedInvite.getUrl()), true);
            if (expandedInvite.getChannel() != null) {
                description.appendLine("Channel: %s".formatted(
                        expandedInvite.getChannel().getType() == ChannelType.TEXT ?
                                "<#%s>".formatted(expandedInvite.getChannel().getId()) : expandedInvite.getChannel().getName()
                ), true);
            }
            description.appendLine("Max Uses: `%s`".formatted(expandedInvite.getMaxUses() == 0 ? "Infinite" : expandedInvite.getMaxUses()), true);
            description.appendLine("Creator: %s".formatted(expandedInvite.getInviter() == null ? "Unknown" : expandedInvite.getInviter().getAsMention()), true);;
            logFeature.sendLog(guild, LogType.INVITE_CREATE, EmbedUtils.successEmbed()
                    .setDescription(description
                            .build())
                    .build());
        });
    }
}
