package cc.fascinated.bat.features.scoresaber;

import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.scoresaber.profile.guild.UserScoreFeedProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberLeaderboardToken;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberPlayerScoreToken;
import cc.fascinated.bat.model.token.beatsaber.scoresaber.ScoreSaberScoreToken;
import cc.fascinated.bat.service.DiscordService;
import cc.fascinated.bat.service.FeatureService;
import cc.fascinated.bat.service.GuildService;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.Guild;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2(topic = "UserScoreFeed Listener")
public class UserScoreFeedListener implements EventListener {
    private final GuildService guildService;
    private final FeatureService featureService;

    @Autowired
    public UserScoreFeedListener(@NonNull GuildService guildService, @NonNull FeatureService featureService) {
        this.guildService = guildService;
        this.featureService = featureService;
    }

    @Override
    public void onScoresaberScoreReceived(@NotNull ScoreSaberPlayerScoreToken score, @NotNull ScoreSaberLeaderboardToken leaderboard,
                                          @NotNull ScoreSaberScoreToken.LeaderboardPlayerInfo player) {
        for (Guild guild : DiscordService.JDA.getGuilds()) {
            BatGuild batGuild = guildService.getGuild(guild.getId());
            if (batGuild == null) {
                continue;
            }
            ScoreSaberFeature scoreSaberFeature = featureService.getFeature(ScoreSaberFeature.class);
            if (!batGuild.getFeatureProfile().isFeatureEnabled(scoreSaberFeature)) { // Check if the feature is enabled
                return;
            }
            UserScoreFeedProfile profile = batGuild.getProfile(UserScoreFeedProfile.class);
            if (profile == null || profile.getChannelId() == null || !profile.getTrackedUsers().contains(player.getId())) {
                continue;
            }

            TextChannel channel = profile.getTextChannel();
            if (channel == null) {
                continue;
            }
            channel.sendMessageEmbeds(ScoreSaberFeature.buildScoreEmbed(score)).queue();
        }
    }
}
